import os
import sys

import django
from django.conf import settings
from django.core.management import call_command
from django.test.utils import get_runner


if __name__ == '__main__':
    os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'
    django.setup()
    call_command('makemigrations', 'paste')
    TestRunner = get_runner(settings)
    failures = TestRunner().run_tests(['tests'])
    sys.exit(bool(failures))



from contextlib import contextmanager

from django.contrib.auth import get_user_model

from paste import constants
from paste.models import Snippet


@contextmanager
def constant(name, value=True):
    """Set app constant to given value, reverting the change when cleaning up.
    """
    if name is not None:
        original = getattr(constants, name)
        setattr(constants, name, value)
    try:
        yield
    finally:
        if name is not None:
            setattr(constants, name, original)


def create_snippet(content, **kwargs):
    """Create and return a snippet, given its content and any possible kwargs.
    """
    return Snippet.objects.create(content=content, **kwargs)


def create_user(*args, **kwargs):
    """Create and return a user."""
    User = get_user_model()
    return User.objects.create_user(*args, **kwargs)



import paste


project = 'django-paste'

copyright = '2020, Aristotelis Mikropoulos'

author = 'Aristotelis Mikropoulos'

release = paste.__version__

version = release.rsplit('.', 1)[0]

exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

html_theme = 'sphinx_rtd_theme'


def setup(app):
    app.add_object_type(
        'confval', 'confval', objname='configuration value',
        indextemplate='pair: %s; configuration value')



from django.urls import reverse

from rest_framework import status
from rest_framework.test import APITestCase

from tests.mixins import SnippetListTestCaseMixin
from tests.utils import constant, create_snippet, create_user


class SnippetUserTestCase(SnippetListTestCaseMixin, APITestCase):
    """Tests for the user snippet list view."""
    not_allowed = SnippetListTestCaseMixin.not_allowed + ['post']

    def setUp(self):
        """Create and store a dummy snippet."""
        self.snippet = create_snippet('foo', owner=self.user)

    def url(self, pk=1):
        """Return the user snippet list URL, for the given user pk."""
        return reverse('snippet-user', kwargs={'pk': pk})

    def test_get_success(self):
        """User snippet list GET must return the queried user's viewable
        snippets.
        """
        snippet = create_snippet('bar', owner=self.user)
        create_snippet('baz')
        create_snippet('test', owner=self.staff_user)
        response = self.get()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)
        self.assertEqual(response.data[0]['id'], self.snippet.pk)
        self.assertEqual(response.data[1]['id'], snippet.pk)

    def test_get_private(self):
        """User snippet list GET must return private snippets only to those
        authorized to view them.
        """
        create_snippet('foo', private=True, owner=self.user)
        some_user = create_user('someuser')
        expected = [1, 2, 2, 1]

        def check(i):
            response = self.get()
            self.assertEqual(len(response.data), expected[i])

        self.check_for_users(check, some_user)

    def test_get_list_foreign(self):
        """User snippet list GET must not return snippets owned by other users
        if the LIST_FOREIGN setting is True, unless requested by a staff user.
        """
        user = create_user('someuser')
        create_snippet('test')
        expected = [0, 1, 1, 0]

        def check(i):
            response = self.get()
            self.assertEqual(len(response.data), expected[i])

        with constant('LIST_FOREIGN', False):
            self.check_for_users(check, user)

    def test_pagination(self):
        """User snippet list must be able to handle pagination."""
        self.check_pagination(owner=self.user)



from typing import List, Tuple

from django.contrib.auth import get_user_model
from django.db import models
from django.utils.translation import gettext_lazy as _

from pygments.lexers import get_all_lexers
from pygments.styles import get_all_styles

from paste import constants


_lexers = (item for item in get_all_lexers() if item[1])


def _max_len(choices: List[Tuple[str, str]]) -> int:
    """Return the maximum length of the first subitems of each item."""
    return max(len(item[0]) for item in choices)


class Snippet(models.Model):
    """A source code snippet with its highlighting and privacy options. May be
    owned by a user.
    """
    LANGUAGE_CHOICES = sorted([(item[1][0], item[0]) for item in _lexers])
    STYLE_CHOICES = sorted([(item, item) for item in get_all_styles()])

    title = models.CharField(
        _('title'), max_length=constants.TITLE_MAX_LENGTH, blank=True)
    content = models.TextField(_('content'))
    language = models.CharField(
        _('language'), choices=LANGUAGE_CHOICES,
        max_length=_max_len(LANGUAGE_CHOICES), blank=True)
    style = models.CharField(
        _('style'), choices=STYLE_CHOICES,
        max_length=_max_len(STYLE_CHOICES), blank=True)
    line_numbers = models.BooleanField(
        _('line numbers'), default=constants.DEFAULT_LINE_NUMBERS)
    embed_title = models.BooleanField(
        _('embed title'), default=constants.DEFAULT_EMBED_TITLE)
    private = models.BooleanField(
        _('private'), default=constants.DEFAULT_PRIVATE)
    created = models.DateTimeField(_('created'), auto_now_add=True)
    updated = models.DateTimeField(_('last modified'), auto_now=True)
    owner = models.ForeignKey(
        get_user_model(), verbose_name=_('owner'),
        blank=True, null=True, on_delete=models.CASCADE)

    class Meta:
        verbose_name = _('snippet')
        verbose_name_plural = _('snippets')

    def __str__(self) -> str:
        return f'{self.title or _("Untitled")} ({self.pk})'



from rest_framework import permissions
from rest_framework.request import Request
from rest_framework.viewsets import ViewSet

from paste import constants
from paste.models import Snippet


class SnippetPermissions(permissions.BasePermission):
    """Permissions for snippet-related views."""

    def has_permission(self, request: Request, view: ViewSet) -> bool:
        """Decide regarding current user type and settings for anonymous
        access, listing, anonymous listing and anonymous creation.
        """
        user = request.user
        if constants.FORBID_ANONYMOUS and user.is_anonymous:
            return False

        if view.action in ['list', 'user']:
            if constants.FORBID_LIST:
                return user.is_staff  # type: ignore
            if constants.FORBID_ANONYMOUS_LIST:
                return user.is_authenticated

        return not (
            view.action == 'create'
            and constants.FORBID_ANONYMOUS_CREATE and user.is_anonymous)

    def has_object_permission(
            self, request: Request, view: ViewSet, obj: Snippet) -> bool:
        """Allow if method is safe and snippet is public, or if user is owner
        or staff.
        """
        user = request.user
        return (
            request.method in permissions.SAFE_METHODS and not obj.private
            or obj.owner == user or user.is_staff)  # type: ignore



from django.urls import reverse

from rest_framework import status
from rest_framework.pagination import CursorPagination

from paste.views import SnippetViewSet

from tests.utils import constant, create_snippet, create_user


class SnippetTestCaseMixin:
    """Snippet test case common state and behavior."""
    constants = []

    @classmethod
    def setUpClass(cls):
        """Create and store a dummy user and a dummy staff user."""
        super().setUpClass()
        cls.user = create_user('user')
        cls.staff_user = create_user('staff', is_staff=True)

    def get(self, **kwargs):
        """Send a GET request to the view's URL with given kwargs and return
        the response.
        """
        return self.client.get(self.url(**kwargs))

    def assert_status(self, method_name, status_code, **kwargs):
        """Assert that a request to the view's URL with given kwargs its type
        being specified by given method name, responds with given status code.
        """
        method = getattr(self.client, method_name)
        response = method(self.url(**kwargs))
        self.assertEqual(response.status_code, status_code)

    def check_for_users(self, check, *extra_users):
        """Call given check function while representing an anonymous user, the
        dummy user, the dummy staff user and any given extra users.
        """
        for i, user in enumerate(
                (None, self.user, self.staff_user) + extra_users):
            self.client.force_authenticate(user)
            check(i)
            self.client.force_authenticate(None)

    def test_not_allowed(self):
        """This view must return a 405 Not Allowed response when accessed by
        one of its not-allowed methods.
        """
        for method in self.not_allowed:
            self.assert_status(method, status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_safe_methods(self):
        """This view's safe methods must return an appropriate response
        depending on the currently authenticated user.
        """

        def check(j):
            for k, setting in enumerate(
                    [None, 'FORBID_ANONYMOUS'] + self.constants):
                with constant(setting):
                    self.assert_status(method, self.safe_expected[i][j][k])

        for i, method in enumerate(['get', 'head', 'options']):
            self.check_for_users(check)


class SnippetDetailTestCaseMixin(SnippetTestCaseMixin):
    """Snippet detail-like test case common state and behavior."""
    safe_expected = ([[
        [status.HTTP_200_OK, status.HTTP_403_FORBIDDEN],
        [status.HTTP_200_OK] * 2, [status.HTTP_200_OK] * 2]] * 3)

    def setUp(self):
        """Create a dummy snippet."""
        create_snippet('foobaz bar', title='baz 42!')

    def url(self, pk=1):
        """Return the view's URL, for the given snippet pk."""
        return reverse(f'snippet-{self.name}', kwargs={'pk': pk})


class _SnippetPagination(CursorPagination):
    page_size = 10


class SnippetListTestCaseMixin(SnippetTestCaseMixin):
    """Snippet list-like test case common state and behavior."""
    not_allowed = ['delete', 'patch', 'put', 'trace']
    constants = ['FORBID_ANONYMOUS_LIST', 'FORBID_LIST']
    _list_expected = [
        [status.HTTP_200_OK] + [status.HTTP_403_FORBIDDEN] * 3,
        [status.HTTP_200_OK] * 3 + [status.HTTP_403_FORBIDDEN],
        [status.HTTP_200_OK] * 4]
    _options_expected = [
        [status.HTTP_200_OK] + [status.HTTP_403_FORBIDDEN]
        + [status.HTTP_200_OK] * 2,
        [status.HTTP_200_OK] * 4,
        [status.HTTP_200_OK] * 4]
    safe_expected = [_list_expected] * 2 + [_options_expected]

    def check_pagination(self, **kwargs):
        """Check that this view is able to handle pagination."""
        for i in range(20):
            create_snippet(i, **kwargs)
        SnippetViewSet.pagination_class = _SnippetPagination
        try:
            response = self.get()
            self.assertEqual(len(response.data['results']), 10)
        finally:
            SnippetViewSet.pagination_class = None



from django.urls import include, path

from rest_framework.routers import DefaultRouter

from paste import views


router = DefaultRouter()
router.register('', views.SnippetViewSet)

urlpatterns = [
    path('', include(router.urls)),
]



from django.utils.translation import gettext as _

from rest_framework.test import APITestCase

from paste.constants import _setting

from tests.utils import create_snippet


class SnippetModelTestCase(APITestCase):
    """Tests for the snippet model."""

    def test_str_titled(self):
        """A titled snippet's string representation must include its title and
        its pk.
        """
        snippet = create_snippet('foo', title='bar')
        self.assertEqual(str(snippet), f'bar ({snippet.pk})')

    def test_str_untitled(self):
        """An untitled snippet's string representation must include the
        localized string "Untitled" and its pk.
        """
        snippet = create_snippet('foo')
        self.assertEqual(str(snippet), f'{_("Untitled")} ({snippet.pk})')


class SettingsTestCase(APITestCase):
    """Tests for the app settings."""

    def test_no_dict(self):
        """Setting must have its requested default value if no PASTE dict is
        defined in project settings.
        """
        value = _setting('FORBID_ANONYMOUS', False)
        self.assertFalse(value)

    def test_no_key(self):
        """Setting must have its requested default value if no related key
        exists in the PASTE dict of project settings.
        """
        with self.settings(PASTE={'GUESS_LEXER': True}):
            value = _setting('FORBID_ANONYMOUS', False)
        self.assertFalse(value)

    def test_override(self):
        """Setting must have the value of the item of the project settings'
        PASTE dict with the same key.
        """
        with self.settings(PASTE={'FORBID_ANONYMOUS': True}):
            value = _setting('FORBID_ANONYMOUS', False)
        self.assertTrue(value)



from rest_framework import serializers

from paste import constants
from paste.models import Snippet


class SnippetSerializer(serializers.ModelSerializer):
    """Snippet model serializer."""

    class Meta:
        model = Snippet
        fields = '__all__'
        read_only_fields = ['owner']

    def create(self, validated_data: dict) -> Snippet:
        """Check that if current user is anonymous they are not trying to
        create a private snippet, then create new instance.
        """
        if (self.context['request'].user.is_anonymous
                and validated_data.get('private', constants.DEFAULT_PRIVATE)):
            raise serializers.ValidationError(
                'anonymous users cannot create private snippets')

        return super().create(validated_data)



from django.contrib.auth import get_user_model
from django.db.models import Q, QuerySet

from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.generics import get_object_or_404
from rest_framework.renderers import StaticHTMLRenderer
from rest_framework.response import Response

from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, guess_lexer

from paste import constants
from paste.models import Snippet
from paste.permissions import SnippetPermissions
from paste.serializers import SnippetSerializer


class SnippetViewSet(viewsets.ModelViewSet):
    """Snippet-related views.

    - Snippet list: / (GET/POST)
    - Snippet detail: /{snippet-id}/ (GET/PUT/PATCH/DELETE)
    - User snippet list: /user/{user-id}/ (GET)
    - Snippet highlight: /{snippet-id}/highlight/ (GET)
    """
    queryset = Snippet.objects.all()
    serializer_class = SnippetSerializer
    permission_classes = [SnippetPermissions]

    def get_queryset(self) -> QuerySet:
        """If current user is staff return all snippets. Else, return those
        owned by current user and, if the relative setting allows so, all the
        public ones.
        """
        queryset = super().get_queryset()
        user = self.request.user
        if user.is_staff:
            return queryset

        query = Q(private=False) if constants.LIST_FOREIGN else Q(pk__in=[])

        if user.is_authenticated:
            query |= Q(owner=user)

        return queryset.filter(query)

    def perform_create(self, serializer: SnippetSerializer) -> None:
        """Store current user if authenticated, then create new instance."""
        kwargs = {}

        user = self.request.user
        if user.is_authenticated:
            kwargs['owner'] = user

        serializer.save(**kwargs)

    @action(detail=True, renderer_classes=[StaticHTMLRenderer])
    def highlight(self, *args, **kwargs) -> Response:
        """Highlight and return the snippet's content as HTML."""
        instance = self.get_object()

        if instance.language:
            lexer = get_lexer_by_name(instance.language)
        elif constants.GUESS_LEXER:
            lexer = guess_lexer(instance.content)
        else:
            lexer = get_lexer_by_name(constants.DEFAULT_LANGUAGE)

        style = instance.style or constants.DEFAULT_STYLE
        options = (
            {'title': instance.title} if instance.title
            and instance.embed_title else {})
        formatter = HtmlFormatter(
            style=style, full=True, linenos=instance.line_numbers, **options)

        return Response(highlight(instance.content, lexer, formatter))

    @action(detail=False, url_path='user/(?P<pk>[^/.]+)')
    def user(self, *args, pk: str, **kwargs) -> Response:
        """Return snippets belonging to user indicated by the ID in the URL.
        """
        User = get_user_model()
        owner = get_object_or_404(User, pk=pk)
        user_snippets = self.get_queryset().filter(owner=owner)

        page = self.paginate_queryset(user_snippets)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(user_snippets, many=True)
        return Response(serializer.data)



import json

from rest_framework import status
from rest_framework.test import APITestCase

from paste import constants

from tests.mixins import SnippetDetailTestCaseMixin
from tests.utils import create_snippet, create_user


class SnippetDetailTestCase(SnippetDetailTestCaseMixin, APITestCase):
    """Tests for the snippet detail view."""
    name = 'detail'
    not_allowed = ['post', 'trace']

    def request(self, method_name, **kwargs):
        """Create a dummy snippet owned by the dummy user, authenticate as the
        dummy user, then send a request to the view's URL its type being
        specified by given method name, with given kwargs, as JSON, using the
        proper content-type and return the response.
        """
        if kwargs:
            kwargs = {'data': json.dumps(kwargs)}
        snippet = create_snippet(
            'content', title='title',
            language='python', embed_title=False, owner=self.user)
        self.client.force_authenticate(self.user)
        method = getattr(self.client, method_name)
        return method(
            self.url(pk=snippet.pk),
            content_type='application/json', **kwargs)

    def test_not_found(self):
        """Snippet detail must return a 404 Not Found response when queried
        with a non-existent snippet pk.
        """
        for method in ['delete', 'get', 'head', 'patch', 'put']:
            self.assert_status(method, status.HTTP_404_NOT_FOUND, pk=2)

    def test_unsafe_methods(self):
        """Snippet detail unsafe methods must return an appropriate response
        depending on the currently authenticated user.
        """
        owner = create_user('owner')
        expected = [
            [status.HTTP_403_FORBIDDEN] * 2
            + [status.HTTP_204_NO_CONTENT] * 2,
            [status.HTTP_403_FORBIDDEN] * 2 + [status.HTTP_200_OK] * 2,
            [status.HTTP_403_FORBIDDEN] * 2
            + [status.HTTP_400_BAD_REQUEST] * 2]

        def check(j):
            snippet = create_snippet('baz', owner=owner)
            self.assert_status(method, expected[i][j], pk=snippet.pk)
            snippet.delete()

        for i, method in enumerate(['delete', 'patch', 'put']):
            self.check_for_users(check, owner)

    def test_get_success(self):
        """Snippet detail GET must return the queried snippet."""
        response = self.request('get')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['content'], 'content')
        self.assertEqual(response.data['title'], 'title')
        self.assertEqual(response.data['language'], 'python')
        self.assertEqual(response.data['style'], '')
        self.assertEqual(
            response.data['line_numbers'], constants.DEFAULT_LINE_NUMBERS)
        self.assertFalse(response.data['embed_title'])
        self.assertEqual(response.data['private'], constants.DEFAULT_PRIVATE)
        self.assertEqual(response.data['owner'], self.user.pk)

    def test_get_private(self):
        """Snippet detail GET must return the private queried snippet only to
        those authorized to view it.
        """
        owner = create_user('owner')
        snippet = create_snippet('testing', private=True, owner=owner)
        expected = [status.HTTP_404_NOT_FOUND] * 2 + [status.HTTP_200_OK] * 2

        def check(i):
            self.assert_status('get', expected[i], pk=snippet.pk)

        self.check_for_users(check, owner)

    def test_patch_success(self):
        """Snippet detail PATCH must partially update the queried snippet."""
        response = self.request('patch', title='foobar', line_numbers=False)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['content'], 'content')
        self.assertEqual(response.data['title'], 'foobar')
        self.assertEqual(response.data['language'], 'python')
        self.assertEqual(response.data['style'], '')
        self.assertFalse(response.data['line_numbers'])
        self.assertFalse(response.data['embed_title'])
        self.assertEqual(response.data['private'], constants.DEFAULT_PRIVATE)
        self.assertEqual(response.data['owner'], self.user.pk)

    def test_put_success(self):
        """Snippet detail PUT must update the queried snippet."""
        response = self.request('put', content='test', private=True)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['content'], 'test')
        self.assertEqual(response.data['title'], 'title')
        self.assertEqual(response.data['language'], 'python')
        self.assertEqual(response.data['style'], '')
        self.assertEqual(
            response.data['line_numbers'], constants.DEFAULT_LINE_NUMBERS)
        self.assertFalse(response.data['embed_title'])
        self.assertTrue(response.data['private'])
        self.assertEqual(response.data['owner'], self.user.pk)

    def test_put_no_content(self):
        """Snippet detail PUT must return a 400 Bad Request response if no
        content field is set.
        """
        response = self.request('put', style='friendly')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



from html.parser import HTMLParser

from rest_framework import status
from rest_framework.test import APITestCase

from tests.mixins import SnippetDetailTestCaseMixin
from tests.utils import constant, create_snippet


class _SnippetParser(HTMLParser):
    """Saves the text of the HTML document."""

    def __init__(self, *args, **kwargs):
        """Initialize text buffer."""
        super().__init__(*args, **kwargs)
        self.text = []

    def handle_data(self, data):
        """Add data to text buffer."""
        self.text.append(data)

    def get_text(self):
        """Return document's text."""
        return ''.join(self.text)


def _response_text(response):
    """Parse given response's HTML and return its text."""
    parser = _SnippetParser()
    parser.feed(response.data)
    return parser.get_text()


class SnippetHighlightTestCase(SnippetDetailTestCaseMixin, APITestCase):
    """Tests for the snippet highlight view."""
    name = 'highlight'
    not_allowed = ['delete', 'patch', 'post', 'put', 'trace']

    def check_response(self, response, content):
        """Check that given response has a 200 OK status code, an HTML
        content-type and given content exists in its contents.
        """
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertTrue(response['Content-Type'].startswith('text/html'))
        self.assertIn(content, _response_text(response))

    def test_get_success(self):
        """Snippet highlight GET must return an HTML page of the queried
        snippet's highlighted contents.
        """
        response = self.get()
        self.check_response(response, 'foobaz bar')

    def test_title_include(self):
        """Snippet highlight GET must return a response containing the queried
        snippet's title if its embed_title is True.
        """
        response = self.get()
        self.assertIn('baz 42!', _response_text(response))

    def test_title_exclude(self):
        """Snippet highlight GET must return a response not containing the
        queried snippet's title if its embed_title is False.
        """
        snippet = create_snippet(
            'example', title='baz 43!', embed_title=False)
        response = self.get(pk=snippet.pk)
        self.assertNotIn('baz 43!', _response_text(response))

    def test_language(self):
        """Snippet highlight GET must return an HTML page of the queried
        snippet's highlighted contents if its language is set.
        """
        snippet = create_snippet('print("hello")', language='python')
        response = self.get(pk=snippet.pk)
        self.check_response(response, 'print("hello")')

    def test_default_language(self):
        """Snippet highlight GET must return an HTML page of the queried
        snippet's highlighted contents using the default language, if its
        language is not set and the GUESS_LEXER setting is False.
        """
        with constant('GUESS_LEXER', False):
            response = self.get()
        self.check_response(response, 'foobaz bar')



SECRET_KEY = ' '

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'rest_framework',
    'paste.apps.PasteConfig',
]

ROOT_URLCONF = 'tests.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}



import json

from django.urls import reverse

from rest_framework import status
from rest_framework.test import APITestCase

from paste import constants

from tests.mixins import SnippetListTestCaseMixin
from tests.utils import constant, create_snippet, create_user


class SnippetListTestCase(SnippetListTestCaseMixin, APITestCase):
    """Tests for the snippet list view."""

    def url(self):
        """Return the snippet list URL."""
        return reverse('snippet-list')

    def post(self, **kwargs):
        """Send a POST request to the view's URL with data indicated by given
        kwargs, as JSON, using the proper content-type and return the
        response.
        """
        return self.client.post(
            self.url(), data=json.dumps(kwargs),
            content_type='application/json')

    def test_get_success(self):
        """Snippet list GET must return all the viewable snippets."""
        create_snippet('foo')
        create_snippet('bar')
        response = self.get()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)
        self.assertEqual(response.data[0]['content'], 'foo')
        self.assertEqual(response.data[1]['content'], 'bar')

    def test_get_private(self):
        """Snippet list GET must return private snippets only to those
        authorized to view them.
        """
        owner = create_user('owner')
        create_snippet('foo', private=True, owner=owner)
        expected = [0, 0, 1, 1]

        def check(i):
            response = self.get()
            self.assertEqual(len(response.data), expected[i])

        self.check_for_users(check, owner)

    def test_get_list_foreign(self):
        """Snippet list GET must not return snippets owned by other users if
        the LIST_FOREIGN setting is True, unless requested by a staff user.
        """
        create_snippet('foo')
        create_snippet('bar', owner=self.user)
        expected = [0, 1, 2]

        def check(i):
            response = self.get()
            self.assertEqual(len(response.data), expected[i])

        with constant('LIST_FOREIGN', False):
            self.check_for_users(check)

    def test_post_success(self):
        """Snippet list POST must create a new snippet."""
        response = self.post(
            content='foo', style='friendly', embed_title=False)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['content'], 'foo')
        self.assertEqual(response.data['title'], '')
        self.assertEqual(response.data['language'], '')
        self.assertEqual(response.data['style'], 'friendly')
        self.assertEqual(
            response.data['line_numbers'], constants.DEFAULT_LINE_NUMBERS)
        self.assertFalse(response.data['embed_title'])
        self.assertEqual(response.data['private'], constants.DEFAULT_PRIVATE)
        self.assertIsNone(response.data['owner'])

    def test_post_owner(self):
        """Snippet list POST must store currently authenticated user as the
        newly created snippet's owner.
        """
        self.client.force_authenticate(self.user)
        response = self.post(content='foo')
        self.assertEqual(response.data['owner'], self.user.pk)

    def test_post_no_content(self):
        """Snippet list POST must return a 400 Bad Request response if no
        content field is set.
        """
        response = self.post(title='foo')
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_oversized_title(self):
        """Snippet list POST must return a 400 Bad Request response if the
        title field consists of more characters than the TITLE_MAX_LENGTH
        setting indicates.
        """
        title = 'a' * (constants.TITLE_MAX_LENGTH + 1)
        response = self.post(content='foo', title=title)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_invalid(self):
        """Snippet list POST must return a 400 Bad Request response if a value
        different than the available choices is set for a multiple choice
        field.
        """
        for field in ['language', 'style']:
            response = self.post(
                **{'content': 'foo', field: '123-invalid-abc'})
            self.assertEqual(
                response.status_code, status.HTTP_400_BAD_REQUEST)

    def check_post_forbid_anonymous(self, setting):
        """Check that snippet list POST returns a 403 Forbidden response to
        anonymous users if the given setting is True.
        """
        expected = (
            [status.HTTP_403_FORBIDDEN] + [status.HTTP_400_BAD_REQUEST] * 2)

        def check(i):
            response = self.post()
            self.assertEqual(response.status_code, expected[i])

        with constant(setting):
            self.check_for_users(check)

    def test_post_forbid_anonymous(self):
        """Snippet list POST must return a 403 Forbidden response to anonymous
        users if the FORBID_ANONYMOUS setting is True.
        """
        self.check_post_forbid_anonymous('FORBID_ANONYMOUS')

    def test_post_forbid_anonymous_create(self):
        """Snippet list POST must return a 403 Forbidden response to anonymous
        users if the FORBID_ANONYMOUS_CREATE setting is True.
        """
        self.check_post_forbid_anonymous('FORBID_ANONYMOUS_CREATE')

    def test_post_anonymous_private(self):
        """Snippet list POST must return a 400 Bad Request response to
        anonymous users who attempt to create a private snippet.
        """
        response = self.post(content='foo', private=True)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_pagination(self):
        """Snippet list must be able to handle pagination."""
        self.check_pagination()



import re
from pathlib import Path

from setuptools import setup


init_path = Path('paste') / '__init__.py'
with init_path.open() as f:
    version = re.search(r'__version__ = \'(.*?)\'', f.read()).group(1)

with open('README.rst') as f:
    long_description = f.read()


setup(
    name='django-paste',
    version=version,
    description='Pluggable, configurable, pastebin HTTP REST API',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Aristotelis Mikropoulos',
    author_email='amikrop@gmail.com',
    url='https://github.com/amikrop/django-paste',
    packages=['paste'],
    license='MIT',
    install_requires=[
        'Django',
        'djangorestframework',
        'Pygments',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



from typing import TypeVar

from django.conf import settings


T = TypeVar('T', bool, int, str)


def _setting(name: str, default: T) -> T:
    """Get value from the app's settings dict, or given default on failure."""
    try:
        settings_dict = getattr(settings, 'PASTE')
    except AttributeError:
        return default
    return settings_dict.get(name, default)


DEFAULT_EMBED_TITLE: bool = _setting('DEFAULT_EMBED_TITLE', True)

DEFAULT_LANGUAGE: str = _setting('DEFAULT_LANGUAGE', 'text')

DEFAULT_LINE_NUMBERS: bool = _setting('DEFAULT_LINE_NUMBERS', True)

DEFAULT_PRIVATE: bool = _setting('DEFAULT_PRIVATE', False)

DEFAULT_STYLE: str = _setting('DEFAULT_STYLE', 'default')

FORBID_ANONYMOUS: bool = _setting('FORBID_ANONYMOUS', False)

FORBID_ANONYMOUS_CREATE: bool = _setting('FORBID_ANONYMOUS_CREATE', False)

FORBID_ANONYMOUS_LIST: bool = _setting('FORBID_ANONYMOUS_LIST', False)

FORBID_LIST: bool = _setting('FORBID_LIST', False)

GUESS_LEXER: bool = _setting('GUESS_LEXER', True)

LIST_FOREIGN: bool = _setting('LIST_FOREIGN', True)

TITLE_MAX_LENGTH: int = _setting('TITLE_MAX_LENGTH', 100)
